@@ -1179,9 +1179,8 @@ bonobo_object_directory_re_check_fn (const Bonobo_ActivationEnvironment *environ
 void
 bonobo_object_directory_reload (void)
 {
-#ifdef BONOBO_ACTIVATION_DEBUG
-        g_print ("reloading our object directory!\n");
-#endif
+        g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "reloading our object directory!");
+
         update_registry (main_dir, TRUE);
 }
 
@@ -1223,8 +1222,6 @@ object_directory_class_init (ObjectDirectoryClass *klass)
 	epv->dynamic_add_path    = impl_Bonobo_ObjectDirectory_add_path;
 	epv->dynamic_remove_path = impl_Bonobo_ObjectDirectory_remove_path;
         epv->addClientEnv        = impl_Bonobo_ObjectDirectory_addClientEnv;
-
-	epv->register_new        = impl_Bonobo_ObjectDirectory_register_new;
 }
 
 static void
